
unit AHCITypes;
interface
uses 
    util,
    PCI,
    drivertypes,
    drivermanagement,
    lmemorymanager,
    console,
    vmemorymanager;

const



type

    { 
        AHCI Host Bus Adapter (HBA) Memory-Mapped Register Interface
        This structure is used to access the AHCI HBA's memory-mapped registers. 
        The AHCI HBA's memory-mapped registers are used to configure the HBA and
        to issue commands to the SATA devices connected to the HBA.

        The AHCI HBA's memory-mapped registers are accessed by reading and writing
        to the HBA's memory-mapped I/O space. The HBA's memory-mapped I/O space is
        typically mapped to a region of physical memory by the system's BIOS or
        UEFI firmware. The HBA's memory-mapped I/O space is typically mapped to a
        region of physical memory that is accessible to the system's CPU(s) and
        other devices.
    }
    THBA_Port = bitpacked record 
        cmdl_basel: uint32;     // Command List Base Address Lower 32-bits
        cmdl_baseu: uint32;     // Command List Base Address Upper 32-bits
        fis_basel: uint32;      // FIS Base Address Lower 32-bits
        fis_baseu: uint32;      // FIS Base Address Upper 32-bits
        int_status: uint32;     // Interrupt Status
        int_enable: uint32;     // Interrupt Enable
        cmd: uint32;            // Command and Status
        rsv0: uint32;           // Reserved
        tfd: uint32;            // Task File Data
        signature: uint32;      // Signature
        sata_stat: uint32;      // SATA Status (SCR0:SStatus)
        sata_ctrl: uint32;      // SATA Control (SCR2:SControl)
        sata_error: uint32;     // SATA Error (SCR1:SError)
        sata_active: uint32;    // SATA Active
        cmd_issue: uint32;      // Command Issue
        sata_noti: uint32;      // SATA Notification
        fis_switch_ctrl: uint32;// FIS-based Switch Control
        rsv1: array[0..10] of uint32;
    end;

    { 
        AHCI Host Bus Adapter (HBA) Memory-Mapped Register Interface
        This structure is used to access the AHCI HBA's memory-mapped registers. 
        The AHCI HBA's memory-mapped registers are used to configure the HBA and
        to issue commands to the SATA devices connected to the HBA.
    }
    THBA_Memory = bitpacked record 
        capabilites: uint32;    // Host Capabilities
        global_ctrl: uint32;    // Global Host Control
        int_status: uint32;     // Interrupt Status
        ports_implemented: uint32; // Ports Implemented
        version: uint32;        // Version
        ccc_control: uint32;    // Command Completion Coalescing Control
        ccc_ports: uint32;      // Command Completion Coalescing Ports
        em_location: uint32;    // Enclosure Management Location
        em_control: uint32;     // Enclosure Management Control
        capabilites2: uint32;   // Host Capabilities Extended
        bohc: uint32;           // BIOS/OS Handoff Control and Status
        rsv0: array[0..0x210] of boolean;
        ports: array[0..31] of THBA_Port;
    end;

    { 
        AHCI Host Bus Adapter (HBA) FIS (Frame Information Structure) Interface
        This structure is used to access the AHCI HBA's FIS (Frame Information Structure)
        memory-mapped registers. The AHCI HBA's FIS memory-mapped registers are used to
        configure the HBA and to issue commands to the SATA devices connected to the HBA.
    }
    THBA_FIS = bitpacked record 
        dsfis: array[0..0x1F] of uint32; // DMA Setup FIS
        rsv0: array[0..0x1F] of uint32;
        psfis: array[0..0x1F] of uint32; // PIO Setup FIS
        rsv1: array[0..0x1F] of uint32;
        rfis: array[0..0x1F] of uint32;  // D2H Register FIS
        rsv2: array[0..0x1F] of uint32;
        sdbfis: array[0..0xF] of uint32; // Set Device Bits FIS
        ufis: array[0..0x1F] of uint32;  // Unknown FIS
        rsv3: array[0..0x67] of uint32;
    end;

    { 
        AHCI Host Bus Adapter (HBA) Command Header Interface
        This structure is used to access the AHCI HBA's Command Header memory-mapped
        registers. The AHCI HBA's Command Header memory-mapped registers are used to
        configure the HBA and to issue commands to the SATA devices connected to the HBA.
    }
    THBA_CMD_HEADER = bitpacked record 
        cmd_fis_length: uint8;   // Command FIS Length
        atapi: uint8;            // ATAPI
        write: uint8;            // Write
        prefetchable: uint8;     // Prefetchable
        reset: uint8;            // Reset
        bist: uint8;             // BIST
        clear_busy: uint8;       // Clear Busy
        reserved0: uint8;        // Reserved
        prdtl: uint16;           // Physical Region Descriptor Table Length
        prdbc: uint32;           // Physical Region Descriptor Byte Count
        cmd_table_base: uint32;  // Command Table Base Address
        cmd_table_baseu: uint32; // Command Table Base Address Upper 32-bits
        rsv0: array[0..4] of uint32;
    end;

    { 
        AHCI Host Bus Adapter (HBA) Command Table Interface
        This structure is used to access the AHCI HBA's Command Table memory-mapped
        registers. The AHCI HBA's Command Table memory-mapped registers are used to
        configure the HBA and to issue commands to the SATA devices connected to the HBA.
    }
    THBA_CMD_TABLE = bitpacked record 
        cmd_fis: THBA_FIS;       // Command FIS
        acmd: array[0..0x1F] of uint8; // ATAPI Command
        rsv0: array[0..0x30] of uint8;
    end;

    { 
        AHCI Host Bus Adapter (HBA) Command Table Interface
        This structure is used to access the AHCI HBA's Command Table memory-mapped
        registers. The AHCI HBA's Command Table memory-mapped registers are used to
        configure the HBA and to issue commands to the SATA devices connected to the HBA.
    }
    THBA_PRD = bitpacked record 
        dba: uint32;             // Data Base Address
        dbau: uint32;            // Data Base Address Upper 32-bits
        rsv0: uint32;            // Reserved
        dbc: uint32;             // Data Byte Count
        rsv1: uint32;            // Reserved
    end;

    { 
        AHCI Host Bus Adapter (HBA) Command Table Interface
        This structure is used to access the AHCI HBA's Command Table memory-mapped
        registers. The AHCI HBA's Command Table memory-mapped registers are used to
        configure the HBA and to issue commands to the SATA devices connected to the HBA.
    }
    THBA_CMD = bitpacked record 
        header: THBA_CMD_HEADER; // Command Header
        table: THBA_CMD_TABLE;   // Command Table
        prd: array[0..0x7] of THBA_PRD; // Physical Region Descriptor Table
    end;

    { 
        AHCI Host Bus Adapter (HBA) Command Table Interface
        This structure is used to access the AHCI HBA's Command Table memory-mapped
        registers. The AHCI HBA's Command Table memory-mapped registers are used to
        configure the HBA and to issue commands to the SATA devices connected to the HBA.
    }
    THBA = bitpacked record 
        memory: THBA_Memory; // HBA Memory
        cmd: array[0..0x7FF] of THBA_CMD; // Command List
    end;

    PHBA = ^THBA;



implementation

end.


